# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Curations ui views module."""

from typing import cast

from flask import Blueprint, Flask, abort, g, render_template
from flask_login import current_user
from flask_principal import Identity
from invenio_records_resources.services.records.service import RecordService
from invenio_users_resources.proxies import current_user_resources

from ..proxies import current_curations_service, unproxy
from ..searchapp import search_app_context
from ..services import CurationRequestService


def user_has_curations_management_role(identity: Identity) -> bool:
    """Check if provided identity provides the curation role."""
    _curations_service: CurationRequestService = unproxy(current_curations_service)
    role = _curations_service.moderation_role
    if not role:
        return False
    return cast(bool, identity.user.has_role(role))


def curation_requests_overview() -> str:
    """Display user dashboard page."""
    if not user_has_curations_management_role(g.identity):
        abort(403)

    user_service_unproxied: RecordService = unproxy(
        current_user_resources.users_service,
    )
    url = user_service_unproxied.links_item_tpl.expand(g.identity, current_user)[
        "avatar"
    ]

    return render_template(
        "invenio_curations/overview.html",
        searchbar_config=dict(searchUrl="/"),
        user_avatar=url,
    )


def create_ui_blueprint(app: Flask) -> Blueprint:  # noqa: ARG001
    """Register blueprint routes on app."""
    blueprint = Blueprint(
        "invenio_curations",
        __name__,
        template_folder="../templates",
        static_folder="../static",
    )

    # Add URL rules
    blueprint.add_url_rule("/curations/overview", view_func=curation_requests_overview)

    # Add context processor for search
    blueprint.app_context_processor(search_app_context)
    return blueprint



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Custom events module."""
from typing import cast

from invenio_requests.customizations.event_types import CommentEventType
from marshmallow import fields


class CurationCommentEventType(CommentEventType):
    """Curation Comment event type."""

    def payload_schema() -> dict:
        """Return payload schema as a dictionary."""
        schema = cast(dict, CommentEventType.payload_schema())

        # reference_draft represents the base of comparison for the content (a custom display of a diff)
        # if the comment needs an update, new data is compared with the data present in this field, thus
        # avoiding the need for draft revisions.
        schema["reference_draft"] = fields.Str(required=False)

        return schema



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Celery tasks for curations."""

from celery import shared_task
from flask import current_app
from invenio_access.permissions import system_identity
from invenio_rdm_records.records.api import RDMDraft

from .proxies import current_curations_service, unproxy
from .services import CurationRequestService
from .services.errors import OpenRecordCurationRequestAlreadyExistsError


@shared_task(ignore_result=True)
def request_record_curation(topic: RDMDraft, user_id: str) -> None:
    """Creates a task to request moderation for a RDM record."""
    _curations_service: CurationRequestService = unproxy(current_curations_service)
    try:
        _curations_service.create(system_identity, topic=topic, user_id=user_id)
    except OpenRecordCurationRequestAlreadyExistsError as ex:
        # Request already exists. Just log the warning.
        current_app.logger.warning(ex.description)



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2016-2021 CERN.
# Copyright (C)      2024 Graz University of Technology.
# Copyright (C)      2024 TU Wien.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS bundles for curations.

You include one of the bundles in a page like the example below (using
``comunities`` bundle as an example):

.. code-block:: html

    {{ webpack['invenio-curations.js'] }}

"""

from invenio_assets.webpack import WebpackThemeBundle

curations = WebpackThemeBundle(
    __name__,
    "assets",
    default="semantic-ui",
    themes={
        "semantic-ui": dict(
            entry={
                "invenio-curations-deposit": "./js/invenio_curations/deposit/index.js",
            },
            dependencies={},
            aliases={
                "@translations/invenio_curations": "translations/invenio_curations",
                "@js/invenio_curations": "js/invenio_curations",
            },
        ),
    },
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021-2022 CERN.
# Copyright (C) 2021-2022 TU Wien.
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Requests resource."""

from typing import Any, cast

from flask import Blueprint, g
from flask_resources import resource_requestctx, response_handler, route
from invenio_records_resources.resources import RecordResource
from invenio_records_resources.resources.records.resource import (
    request_data,
    request_extra_args,
    request_search_args,
    request_view_args,
)
from invenio_records_resources.resources.records.utils import search_preference
from invenio_records_resources.services.records.results import RecordItem, RecordList


#
# Resource
#
class CurationsResource(RecordResource):
    """Resource for curations."""

    def create_blueprint(self, **options: Any) -> Blueprint:
        """Create the blueprint."""
        # We avoid passing url_prefix to the blueprint because we need to
        # install URLs under both /records and /user/records. Instead we
        # add the prefix manually to each route (which is anyway what Flask
        # does in the end)
        options["url_prefix"] = ""
        return cast(Blueprint, super().create_blueprint(**options))

    def as_blueprint(self, **options: Any) -> Blueprint:
        """Creating the blueprint and registering error handlers on the application.

        This is required, as the CurationComponent will throw inside another blueprint.
        """
        blueprint = cast(Blueprint, super().as_blueprint(**options))

        for exc_or_code, error_handler in self.create_error_handlers():
            blueprint.record_once(
                lambda s, exc_or_code=exc_or_code, error_handler=error_handler: s.app.errorhandler(
                    exc_or_code,
                )(
                    error_handler,
                ),
            )

        return blueprint

    def create_url_rules(
        self,
    ) -> list[dict]:
        """Create the URL rules for the record resource."""
        routes = self.config.routes

        def p(route: str) -> str:
            """Prefix a route with the URL prefix."""
            return f"{self.config.url_prefix}{route}"

        def s(route: str) -> str:
            """Suffix a route with the URL prefix."""
            return f"{route}{self.config.url_prefix}"

        return [
            route("GET", p(routes["list"]), self.search),
            route("POST", p(routes["list"]), self.create),
        ]

    @request_extra_args
    @request_search_args
    @request_view_args
    @response_handler(many=True)
    def search(self) -> tuple[dict[str, Any], int]:
        """Perform a search over the items."""
        hits: RecordList = self.service.search(
            identity=g.identity,
            params=resource_requestctx.args,
            search_preference=search_preference(),
            expand=resource_requestctx.args.get("expand", False),
        )
        return hits.to_dict(), 200

    @request_extra_args
    @request_data
    @response_handler()
    def create(self) -> tuple[dict[str, Any], int]:
        """Create an item."""
        item: RecordItem = self.service.create(
            g.identity,
            resource_requestctx.data or {},
            expand=resource_requestctx.args.get("expand", True),
        )
        return item.to_dict(), 201



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 KTH Royal Institute of Technology
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Requests resource config."""

from typing import Final

import marshmallow as ma
from flask_resources import HTTPJSONException, create_error_handler
from invenio_records_resources.resources import RecordResourceConfig
from invenio_records_resources.services.base.config import ConfiguratorMixin, FromConfig
from invenio_requests.resources.requests.config import RequestSearchRequestArgsSchema

from ..services.errors import (
    CurationRequestNotAcceptedError,
    OpenRecordCurationRequestAlreadyExistsError,
    RoleNotFoundError,
)


#
# Request args
#
class CurationsSearchRequestArgsSchema(RequestSearchRequestArgsSchema):
    """Add parameter to parse tags."""


request_error_handlers = {
    OpenRecordCurationRequestAlreadyExistsError: create_error_handler(
        lambda e: HTTPJSONException(
            code=400,
            description=str(e),
        ),
    ),
    CurationRequestNotAcceptedError: create_error_handler(
        lambda e: HTTPJSONException(
            code=400,
            description=str(e),
            errors=[
                {
                    "field": "metadata.title",
                    "messages": ["."],
                },
            ],
        ),
    ),
    RoleNotFoundError: create_error_handler(
        lambda e: HTTPJSONException(
            code=404,
            description=str(e),
        ),
    ),
}


#
# Resource config
#
class CurationsResourceConfig(RecordResourceConfig, ConfiguratorMixin):
    """Requests resource configuration."""

    # types differ from superclass
    blueprint_name = "curations"
    url_prefix = "/curations"
    routes: Final = {
        "list": "/",
    }

    request_view_args: Final = {
        "reference_type": ma.fields.Str(),
        "reference_id": ma.fields.Str(),
    }

    # types differ from superclass
    request_extra_args: Final = {
        **RecordResourceConfig.request_extra_args,
        "reference_type": ma.fields.Str(),
        "reference_id": ma.fields.Str(),
    }
    request_search_args = CurationsSearchRequestArgsSchema

    error_handlers = FromConfig(
        "CURATIONS_ERROR_HANDLERS",
        default=request_error_handlers,
    )

    response_handlers: Final = {
        "application/vnd.inveniordm.v1+json": RecordResourceConfig.response_handlers[
            "application/json"
        ],
        **RecordResourceConfig.response_handlers,
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Resources module."""

from .config import CurationsResourceConfig
from .resource import CurationsResource

__all__ = (
    "CurationsResource",
    "CurationsResourceConfig",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
# Copyright (C) 2024 TU Wien.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Component for checking curations."""

from abc import ABC
from typing import Any, cast

import dictdiffer
from flask_principal import Identity
from invenio_access.permissions import system_identity
from invenio_accounts.models import User
from invenio_drafts_resources.services.records.components import ServiceComponent
from invenio_pidstore.models import PIDStatus
from invenio_rdm_records.records.api import RDMDraft, RDMRecord
from invenio_requests.proxies import current_requests_service
from invenio_requests.services import RequestsService

from ..proxies import current_curations_service
from . import CurationRequestService
from .comment import CommentProcessor
from .diff import DiffProcessor
from .errors import CurationRequestNotAcceptedError


def _get_curations_service() -> CurationRequestService:
    return cast(CurationRequestService, current_curations_service)


def _get_requests_service() -> RequestsService:
    return cast(RequestsService, current_requests_service)


def _skip_curations_flow(privileged_roles: list[str], identity: Identity) -> bool:
    """Admins and system can bypass curations workflow.

    Check for actual user roles because identity can be incomplete
    (e.g. see https://github.com/inveniosoftware/invenio-rdm-records/blob/7c47cdcb133b3b88228d5157687514f0ec7eecc4/invenio_rdm_records/fixtures/tasks.py#L41)
    """
    if identity == system_identity:
        return True

    user_id = identity.id
    user = User.query.get(user_id)

    return any(role in privileged_roles for role in user.roles)


class CurationComponent(ServiceComponent, ABC):
    """Service component for access integration."""

    def publish(
        self,
        identity: Identity,
        draft: RDMDraft | None = None,
        record: RDMRecord | None = None,  # noqa: ARG002
        **kwargs: Any,  # noqa: ARG002
    ) -> None:
        """Check if record curation request has been accepted."""
        # The `PIDComponent` takes care of calling `record.register()` which sets the
        # status for `record.pid.status` to "R", but the draft's dictionary data
        # only gets updated via `record.commit()` (which is performed by the `uow`).
        # Thus, if we spot a discrepancy here we can deduce that this is the first time
        # the record gets published.
        if draft is None:
            msg = "Unexpected publish action with undefined draft."
            raise RuntimeError(msg)

        if _skip_curations_flow(_get_curations_service().privileged_roles, identity):
            # configured roles can publish without curation workflow
            return

        has_been_published = (
            draft.pid.status == draft["pid"]["status"] == PIDStatus.REGISTERED
        )
        if has_been_published and _get_curations_service().allow_publishing_edits:
            return

        review_accepted = _get_curations_service().accepted_record(
            system_identity,
            draft,
        )

        if not review_accepted:
            raise CurationRequestNotAcceptedError

    def delete_draft(
        self,
        identity: Identity,  # noqa: ARG002
        draft: RDMDraft | None = None,
        record: RDMRecord | None = None,
        *,
        force: bool = False,  # noqa: ARG002
    ) -> None:
        """Delete a draft."""
        request = _get_curations_service().get_review(
            system_identity,
            draft,
            expand=True,
        )

        # No open request. Nothing to do.
        if request is None:
            return

        # New record or new version -> request can be removed.
        if record is None:
            _get_requests_service().delete(system_identity, request["id"], uow=self.uow)
            return

        # Delete draft for a published record.
        # Since only one request per record should exist, it is not deleted. Instead, put it back to accepted.
        _get_requests_service().execute_action(
            system_identity,
            request["id"],
            "cancel",
            uow=self.uow,
        )

    def _check_update_request(
        self,
        identity: Identity,  # noqa: ARG002
        request: dict,
        data: dict | None = None,
        record: RDMDraft | None = None,
        errors: list[dict] | None = None,  # noqa: ARG002
    ) -> None:
        """Update request title if record title has changed."""
        updated_draft_title = (data or {}).get("metadata", {}).get("title")
        current_draft_title = (record or {}).get("metadata", {}).get("title")

        if current_draft_title != updated_draft_title:
            title = updated_draft_title or record["id"]  # type: ignore[index]
            request["title"] = f"RDM Curation: {title}"

            # Using system identity, to not have to update the default request can_update permission.
            # Data will be checked in the requests service.
            _get_requests_service().update(
                system_identity,
                request["id"],
                request,
                uow=self.uow,
            )

    def _prepare_data(
        self,
        data: dict,
        current_draft: RDMDraft,
    ) -> tuple[dict, dict]:
        """Build and return fresh new maps only with fields that should be compared later in the process.

        Method used in context of comment processing.
        """
        supported_fields = ["metadata", "custom_fields"]
        new_data = {}
        new_crt_draft = {}
        for field in supported_fields:
            new_data[field] = data[field]
            new_crt_draft[field] = current_draft[field]

        return new_data, new_crt_draft

    def _process_comment(
        self,
        data: dict,
        current_draft: RDMDraft,
        request: dict,
        errors: list[dict] | None,
    ) -> None:
        """Prepare and call all needed structures in order to successfully create/update a request comment."""
        prepared_data, prepared_current_draft = self._prepare_data(data, current_draft)

        diff_processor = DiffProcessor(
            configured_elements=current_curations_service.comments_mapping,
            comment_template_file=current_curations_service.comment_template_file,
        )
        comment_processor = CommentProcessor(system_identity, diff_processor)

        comment_processor.process_comment(
            request,
            prepared_data,
            prepared_current_draft,
            errors,
        )

    def update_draft(
        self,
        identity: Identity,
        data: dict | None = None,
        record: RDMDraft | None = None,
        errors: list[dict] | None = None,
    ) -> None:
        """Update draft handler."""
        has_published_record = record is not None and record.is_published
        if has_published_record and _get_curations_service().allow_publishing_edits:
            return

        if _skip_curations_flow(_get_curations_service().privileged_roles, identity):
            return

        request = _get_curations_service().get_review(
            system_identity,
            record,
            expand=True,
        )

        if not request:
            return

        current_draft: RDMDraft = self.service.draft_cls.pid.resolve(
            record["id"],  # type: ignore[index]
            registered_only=False,
        )

        self._check_update_request(
            identity,
            request,
            data=data,
            record=current_draft,
            errors=errors,
        )

        # TODO: Should updates be disallowed if the record/request is currently being reviewed?
        # It could be possible that the record gets updated while a curator performs a review. The curator would be looking at an outdated record and the review might not be correct.

        # If a request is open, it still has to be reviewed eventually.
        if request["is_open"]:
            if current_curations_service.comments_enabled:
                # prepare and process a comment if config is enabled
                self._process_comment(data, current_draft, request, errors)  # type: ignore[arg-type]
            return

        # Compare metadata of current draft and updated draft.

        # Sometimes the metadata differs between the passed `record` and resolved
        # `current_draft` in references (e.g. in the `record` object, the creator's
        # affiliation has an ID & name, but in the `current_draft` it's only the ID)
        # this discrepancy can be removed by resolving or cleaning the relations
        current_draft.relations.clean()
        record.relations.clean()  # type: ignore[union-attr]

        current_data = self.service.schema.dump(
            current_draft,
            context=dict(
                identity=identity,
                pid=current_draft.pid,
                record=current_draft,
            ),
        )
        # Load updated data with service schema
        updated_data = self.service.schema.dump(
            record,
            context=dict(
                identity=identity,
                pid=record.pid,  # type: ignore[union-attr]
                record=record,
            ),
        )
        # TODO: File updates are not picked up. File actions are handled in dedicated files service.
        #       Files service is not configurable per default and we can not add a component there.
        diff = dictdiffer.diff(current_data, updated_data)
        diff_list = list(diff)

        # Request is closed but draft was updated with new data. Put back for review
        if diff_list:
            _get_requests_service().execute_action(
                identity,
                request["id"],
                "resubmit",
                uow=self.uow,
            )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Curation service."""

from typing import Any, cast

from flask import current_app
from flask_principal import Identity
from flask_security import SQLAlchemyUserDatastore
from invenio_access.permissions import system_identity
from invenio_accounts.models import Role
from invenio_accounts.proxies import current_datastore
from invenio_db.uow import UnitOfWork
from invenio_i18n import gettext as _
from invenio_rdm_records.records.api import RDMDraft
from invenio_records_resources.services.records.results import RecordItem, RecordList
from invenio_records_resources.services.uow import unit_of_work
from invenio_requests.customizations.request_types import RequestType
from invenio_requests.proxies import current_request_type_registry
from invenio_requests.registry import TypeRegistry
from invenio_requests.services import RequestsService
from invenio_requests.services.results import ResolverRegistry
from invenio_search.engine import dsl
from werkzeug.datastructures import ImmutableMultiDict

from ..proxies import unproxy
from ..requests import CurationRequest
from .diff import DiffElement
from .errors import OpenRecordCurationRequestAlreadyExistsError, RoleNotFoundError


class CurationRequestService:
    """Service for User Moderation requests."""

    _request_type_registry: TypeRegistry = unproxy(current_request_type_registry)
    _datastore: SQLAlchemyUserDatastore = unproxy(current_datastore)

    def __init__(self, requests_service: RequestsService, **__: Any) -> None:
        """Service initialisation as a sub-service of requests."""
        self.requests_service = requests_service

    @property
    def allow_publishing_edits(self) -> bool:
        """Get the configured value of ``CURATIONS_ALLOW_PUBLISHING_EDITS``."""
        return cast(
            bool,
            current_app.config.get("CURATIONS_ALLOW_PUBLISHING_EDITS", False),
        )

    @property
    def moderation_role_name(self) -> str:
        """Get the configured name of the ``CURATIONS_MODERATION_ROLE``."""
        role = current_app.config["CURATIONS_MODERATION_ROLE"]

        if isinstance(role, Role):
            # As of InvenioRDM v12, the role's name and ID should be the same anyway
            name: str = role.name or role.id
            return name

        # If it's not a role, we assume it's a string
        return cast(str, role)

    @property
    def moderation_role(self) -> str:
        """Get the configured ``CURATIONS_MODERATION_ROLE`` role."""
        return cast(str, self._datastore.find_role(self.moderation_role_name))

    @property
    def request_type_cls(self) -> type[RequestType]:
        """Curation request type."""
        return cast(
            type[RequestType],
            self._request_type_registry.lookup(CurationRequest.type_id),
        )

    @property
    def comments_enabled(self) -> bool:
        """Get the configured value of ``CURATIONS_ENABLE_REQUEST_COMMENTS``."""
        return cast(
            bool,
            current_app.config.get("CURATIONS_ENABLE_REQUEST_COMMENTS", False),
        )

    @property
    def comments_mapping(self) -> list[DiffElement]:
        """Curations specific comment classes."""
        return cast(
            list[DiffElement],
            current_app.config.get("CURATIONS_COMMENTS_CLASSES"),
        )

    @property
    def comment_template_file(self) -> str:
        """Curations specific comment html template."""
        return cast(str, current_app.config.get("CURATIONS_COMMENT_TEMPLATE_FILE"))

    @property
    def privileged_roles(self) -> list[str]:
        """Curations roles that can bypass the curation approvals."""
        return cast(list[str], current_app.config.get("CURATIONS_PRIVILEGED_ROLES"))

    def get_review(
        self,
        identity: Identity,
        topic: RDMDraft,
        **kwargs: Any,
    ) -> dict[str, Any] | None:
        """Get the curation review for a topic."""
        topic_reference: dict = ResolverRegistry.reference_entity(topic)
        # Assume there is only one item in the reference dict
        topic_key, topic_value = next(iter(topic_reference.items()))

        results: RecordList = self.requests_service.search(
            identity,
            extra_filter=dsl.query.Bool(
                "must",
                must=[
                    dsl.Q("term", **{"type": self.request_type_cls.type_id}),
                    dsl.Q("term", **{f"topic.{topic_key}": topic_value}),
                ],
            ),
            **kwargs,
        )

        if results.total == 0:
            return None

        return cast(dict[str, Any], next(results.hits))

    def accepted_record(
        self,
        identity: Identity,
        record: RDMDraft,
    ) -> dict[str, Any] | None:
        """Check if current version of record has been accepted."""
        topic_reference = ResolverRegistry.reference_entity(record)
        # Assume there is only one item in the reference dict
        topic_key, topic_value = next(iter(topic_reference.items()))

        results: RecordList = self.requests_service.search(
            identity,
            extra_filter=dsl.query.Bool(
                "must",
                must=[
                    dsl.Q("term", **{"type": self.request_type_cls.type_id}),
                    dsl.Q("term", **{f"topic.{topic_key}": topic_value}),
                    dsl.Q("term", **{"is_open": False}),
                    dsl.Q("term", **{"status": "accepted"}),
                ],
            ),
        )
        return next(results.hits) if results.total > 0 else None

    @unit_of_work()
    def create(
        self,
        identity: Identity,
        data: dict[str, Any] | None = None,
        uow: UnitOfWork | None = None,
        **kwargs: Any,
    ) -> RecordItem:
        """Create a RDMCuration request and submit it."""
        role = self.moderation_role
        if not role:
            raise RoleNotFoundError(self.moderation_role_name)
        assert role, _(
            "Curation request moderation role must exist to enable record curation requests.",
        )

        # Allowed entities for the request type will be checked in the request service
        topic = ResolverRegistry.resolve_entity_proxy(data.pop("topic", None)).resolve()  # type: ignore[union-attr]
        creator = (
            ResolverRegistry.resolve_entity_proxy(data.pop("created_by")).resolve()  # type: ignore[union-attr]
            if "created_by" in data  # type: ignore[operator]
            else None
        )

        receiver = (
            ResolverRegistry.resolve_entity_proxy(data.pop("receiver", None)).resolve()  # type: ignore[union-attr]
            if "receiver" in data  # type: ignore[operator]
            else role
        )

        default_data = {
            "title": "RDM Curation: {title}".format(
                title=topic.metadata.get("title", topic["id"]),
            ),
        }

        # using system identity to ensure a request is fetched, if it exists. Even if the user would not have access.
        if self.get_review(system_identity, topic):
            raise OpenRecordCurationRequestAlreadyExistsError

        if data:
            default_data.update(data)

        return self.requests_service.create(
            identity,
            default_data,
            self.request_type_cls,
            receiver,
            creator=creator,
            topic=topic,
            uow=uow,
            **kwargs,
        )

    def search(
        self,
        identity: Identity,
        params: ImmutableMultiDict | None = None,
        search_preference: str | None = None,
        *,
        expand: bool = False,
        **kwargs: Any,
    ) -> RecordList:
        """Search for curation requests."""
        return self.requests_service.search(
            identity,
            extra_filter=dsl.query.Bool(
                "must",
                must=[
                    dsl.Q("term", **{"type": self.request_type_cls.type_id}),
                ],
            ),
            params=params,
            search_preference=search_preference,
            expand=expand,
            **kwargs,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Utils module."""

import nh3


class HTMLParseError(Exception):
    """Custom HTML parsing exception."""


def cleanup_html_tags(text: str) -> str:
    """Strip given text of HTML tags using nh3 library."""
    if not nh3.is_html(text):
        return text
    try:
        return nh3.clean(text, tags=set(), attributes={})
    except Exception as e:
        msg = "Could not parse html input"
        raise HTMLParseError(msg) from e



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module for generic and customizable curations."""

from flask import Flask, g
from flask_menu import current_menu
from invenio_i18n import lazy_gettext as _
from invenio_requests.proxies import current_requests_service
from invenio_requests.services import RequestsService

from . import config
from .proxies import unproxy
from .resources import CurationsResource, CurationsResourceConfig
from .services import (
    CurationRequestService,
    CurationsServiceConfig,
)
from .views.ui import user_has_curations_management_role


def _get_requests_service() -> RequestsService:
    return unproxy(current_requests_service)


class ServiceConfigs:
    """Customized service configs."""

    def __init__(self, app: Flask) -> None:
        """Constructs."""
        self._curations: CurationsServiceConfig = CurationsServiceConfig.build(app)

    @property
    def curations(self) -> CurationRequestService:
        """The curation service."""
        return self._curations


def finalize_app(app: Flask) -> None:
    """Finalize app."""
    init_menu(app)


def init_menu(app: Flask) -> None:  # noqa: ARG001
    """Initialize flask menu."""
    user_dashboard = current_menu.submenu("dashboard")
    user_dashboard.submenu("curation-overview").register(
        "invenio_curations.curation_requests_overview",
        text=_("Curation Requests"),
        order=100,
        visible_when=lambda: user_has_curations_management_role(g.identity),
    )


class InvenioCurations:
    """Invenio-Curations extension."""

    def __init__(self, app: Flask | None = None) -> None:
        """Extension initialization."""
        self.curations_service: CurationRequestService | None = None
        self.curations_resource: CurationsResource | None = None
        if app:
            self.init_app(app)

    def init_app(self, app: Flask) -> None:
        """Flask application initialization."""
        self.init_config(app)
        self.init_services(app)
        self.init_resources(app)
        app.extensions["invenio-curations"] = self

    def init_config(self, app: Flask) -> None:
        """Initialize configuration."""
        for k in dir(config):
            if k.startswith("CURATIONS_"):
                app.config.setdefault(k, getattr(config, k))

    def service_configs(self, app: Flask) -> ServiceConfigs:
        """Customized service configs."""
        return ServiceConfigs(app)

    def init_services(self, app: Flask) -> None:
        """Initialize the service and resource for curations."""
        service_configs = self.service_configs(app)

        self.curations_service = CurationRequestService(
            config=service_configs.curations,
            requests_service=_get_requests_service(),
        )

    def init_resources(self, app: Flask) -> None:
        """Init resources."""
        self.curations_resource = CurationsResource(
            service=self.curations_service,
            config=CurationsResourceConfig.build(app),
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module for curation support."""

from .ext import InvenioCurations
from .proxies import (
    current_curations,
    current_curations_service,
)

__version__ = "0.3.1"

__all__ = (
    "InvenioCurations",
    "__version__",
    "current_curations",
    "current_curations_service",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio module for curations."""

from .notifications.builders import (
    CurationRequestAcceptNotificationBuilder,
    CurationRequestCritiqueNotificationBuilder,
    CurationRequestResubmitNotificationBuilder,
    CurationRequestReviewNotificationBuilder,
    CurationRequestSubmitNotificationBuilder,
)
from .services import facets
from .services.diff import DiffDescription

CURATIONS_FACETS = {
    "type": {
        "facet": facets.type,
        "ui": {
            "field": "type",
        },
    },
    "status": {
        "facet": facets.status,
        "ui": {
            "field": "status",
        },
    },
}
"""Invenio requests facets."""


CURATIONS_ALLOW_PUBLISHING_EDITS = False
"""Allow publishing of metadata edits for already published records.

This allows users to modify their record's metadata and publish those changes
without going through another curation review.
The idea here is that the record has already passed through an initial quality check
anyway and it's very unlikely that anybody would update the metadata afterwards in
a way that decreases their quality.
Even if that happens, InvenioRDM stores `revisions` for record metadata.
"""

CURATIONS_TIMELINE_PAGE_SIZE = 15
"""Amount of items per page on the request details timeline"""


CURATIONS_MODERATION_ROLE = "administration-rdm-records-curation"
"""ID of the Role used for record curation."""

CURATIONS_SEARCH_REQUESTS = {
    "facets": ["type", "status"],
    "sort": ["bestmatch", "newest", "oldest"],
}
"""Curation requests search configuration (i.e list of curations requests)"""

CURATIONS_NOTIFICATIONS_BUILDERS = {
    builder.type: builder
    for builder in [
        CurationRequestAcceptNotificationBuilder,
        CurationRequestCritiqueNotificationBuilder,
        CurationRequestResubmitNotificationBuilder,
        CurationRequestReviewNotificationBuilder,
        CurationRequestSubmitNotificationBuilder,
    ]
}
"""Curation related notification builders as map for easy import."""

CURATIONS_ENABLE_REQUEST_COMMENTS = False
"""Enable or disable the generation of diff comments on ``rdm-curation`` requests.

Set this in order to activate the automatic creation/update of request comments
based on the difference found between draft states of an unpublished record
that is in the curation phase.
"""

CURATIONS_COMMENTS_CLASSES = [DiffDescription]
"""Extend curations comment classes for more diff customization.

List with all custom classes defined for rendering a change in a draft field.
"""

CURATIONS_COMMENT_TEMPLATE_FILE = "comment-template.html"
"""Curation comment template file.

Choose a file from the instance's templates folder to render the request comment.
More details in README.
"""

CURATIONS_PRIVILEGED_ROLES = ["administration"]
"""Curation privileged roles.

Users or processes with these roles can bypass curation checks.
Note: this does not apply for community curations. This config should be used just
to allow admins to publish records without having to perform the extra steps necessary
for approval.
Also used for creating rdm-records demo records in testing.
"""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Proxies for accessing the currently instantiated curations extension."""

from typing import cast

from flask import current_app
from werkzeug.local import LocalProxy


# the funny thing is the ignore was necessary after started using ruff, which
# should not interfere with mypy
def unproxy[T](obj: LocalProxy[T]) -> T:  # type: ignore[type-var]
    """Cast to the proxy bound object."""
    return cast(T, obj)


current_curations = LocalProxy(lambda: current_app.extensions["invenio-curations"])
"""Proxy for the instantiated curations extension."""

current_curations_service = LocalProxy(lambda: current_curations.curations_service)
"""Proxy to the instantiated curations service."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Diff handling module."""
import typing
from abc import ABC, abstractmethod
from typing import Any, Final

from flask import current_app, render_template
from invenio_i18n import lazy_gettext as _

from .utils import HTMLParseError, cleanup_html_tags

"""Supported dictdiffer.diff() types for comment rendering."""
type DIFF_TYPE = tuple[str, str, list | tuple]


class DiffError(Exception):
    """Custom exception for diff exceptions."""


class DiffProcessorBase(ABC):
    """Interface for classes that process diffs in context of comment creation/update."""

    @abstractmethod
    def map_and_build_diffs(self, raw_diffs: list[tuple]) -> None:
        """Maps the raw diffs resulted from dictdiffer.diff to specific structures."""

    @abstractmethod
    def to_html(self, *args: Any) -> str:
        """Represent the inner state in HTML."""


class DiffElement:
    """Wrapper class for a diff tuple."""

    def __init__(self, diff: DIFF_TYPE | None = None) -> None:
        """Constructs."""
        self._diff = diff

    def __repr__(self) -> str:
        """Represent the diff."""
        if self._diff is None:
            raise DiffError
        update, key, result = self._diff
        if update != "change":
            return str({key: result})
        old, new = result
        return str({key: {"old": old, "new": new}})

    @property
    def diff(self) -> DIFF_TYPE | None:
        """Returns current diff."""
        return self._diff

    def match_diff_key(self, diff: DIFF_TYPE) -> bool:  # noqa: ARG002
        """Check if the given ``diff`` matches this diff processor.

        Note: The ``diff`` is expected to be a tuple from ``dictdiffer.diff()``.
        """
        return True

    def cleanup(self) -> typing.Self:
        """Cleanup the current diff with changing or removing unwanted data."""
        return self

    def validate(self) -> bool:
        """Validate the current diff. For the base method means just a sanity check."""
        if self._diff is None:
            raise DiffError

        return True


class DiffDescription(DiffElement):
    """Wrapper for the description field diffs of the metadata."""

    key_name = "metadata.description"

    def match_diff_key(self, diff: DIFF_TYPE) -> bool:
        """Override match_diff_key method.

        Match the possible diffs that contain description changes.
        """
        if not super().match_diff_key(diff):
            return False
        _, key, result = diff
        if key == self.key_name:
            return True
        try:
            name, _ = result[0]
        except Exception:  # noqa: BLE001
            return False
        else:
            return f"{key}.{name}" == self.key_name

    def cleanup(self) -> typing.Self:
        """Override cleanup method to strip HTML tags which are found in description diffs."""
        if self._diff is None:
            raise DiffError
        update, key, result = self._diff
        try:
            if isinstance(result, list) and len(result) == 1:
                field, val = result[0]
                new_val = cleanup_html_tags(val).strip()
                self._diff = (update, key, [(field, new_val)])
                return self
            if isinstance(result, tuple):
                old, new = result
                new_old = cleanup_html_tags(old).strip()
                new_new = cleanup_html_tags(new).strip()
                self._diff = (update, key, (new_old, new_new))
                return self
            current_app.logger.error(
                "Could not evaluate diff element %s of class %s",
                self._diff,
                self.__class__.__name__,
            )
        except HTMLParseError as err:
            current_app.logger.error(
                "Could not parse HTML for diff %s and class %s",
                self._diff,
                self.__class__.__name__,
            )
            raise DiffError from err
        else:
            raise DiffError

    def validate(self) -> bool:
        """Validate the description diff for expected structure."""
        _, _, result = self._diff
        return (isinstance(result, list) and len(result) == 1) or isinstance(
            result,
            tuple,
        )


class DiffProcessor(DiffProcessorBase):
    """DiffProcessor class.

    Used in context of requests comments processing.
    Manages the rendering of a list of diffs (which is the data of a comment).


    :ivar _diffs: A list of diff tuples.
    :ivar _configured_elements: Wrapper classes for specific diffs.
    :ivar _comment_template_file: HTML template filename used to render the comment.
    """

    _known_actions: Final[dict] = {
        "resubmit": _("Record was resubmitted for review with the following changes:"),
        "update_while_critiqued": _("Record was updated after changes were requested!"),
        "update_while_review": _(
            "Record was updated! Please check the latest changes.",
        ),
        "default": _("Action triggered comment update"),
    }

    def __init__(
        self,
        diffs: list[DiffElement] | None = None,
        configured_elements: list[type[DiffElement]] | None = None,
        comment_template_file: str | None = None,
    ) -> None:
        """Constructs."""
        self._diffs = diffs
        self._configured_elements = configured_elements
        self._comment_template_file = comment_template_file

    @property
    def diffs(self) -> list[DiffElement] | None:
        """Returns current diff list."""
        return self._diffs

    def _map_one_diff(self, raw_diff: DIFF_TYPE) -> DiffElement:
        """Return the class of a specific diff.

        :param raw_diff: This should be a tuple resulting from a dictdiffer.diff operation.

        :returns: The wrapper class of the given diff.
        """
        if self._configured_elements is None:
            raise DiffError

        for element in self._configured_elements:
            if element().match_diff_key(raw_diff):
                return element(raw_diff)

        return DiffElement(raw_diff)

    def map_and_build_diffs(self, raw_diffs: list[DIFF_TYPE]) -> None:
        """Maps the diffs to their specific wrapper class, instantiates them and adds them to the list.

        :param raw_diffs: This should be a list of tuples resulted from a dictdiffer.diff operation
        """
        self._diffs = []
        for raw_diff in raw_diffs:
            self._diffs.append(self._map_one_diff(raw_diff))

    def _prepare_content(self) -> None:
        """Removes structures that shouldn't be in diffs list."""
        if self._diffs is None:
            raise DiffError

        to_remove = [diff for diff in self._diffs if not diff.validate()]

        for remove in to_remove:
            self._diffs.remove(remove)

    def to_html(self, action: str) -> str:
        """Renders the diffs using the configured template.

        :param action: Action is used to get the title of the comment.

        :returns: The rendered comment.
        """
        if self._diffs is None or self._comment_template_file is None:
            raise DiffError

        if action not in self._known_actions:
            action = "default"

        adds = []
        changes = []
        removes = []

        self._prepare_content()
        for diff in self._diffs:
            # we shouldn't have None returned here after prepare_content
            update, _, _ = diff.cleanup().diff
            if update == "add":
                adds.append(diff)
            elif update == "change":
                changes.append(diff)
            elif update == "remove":
                removes.append(diff)

        try:
            return render_template(
                self._comment_template_file,
                adds=adds,
                changes=changes,
                removes=removes,
                header=self._known_actions[action],
            )
        except Exception as e:
            raise DiffError from e



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Curations permissions."""

from typing import Final

from invenio_rdm_records.requests import CommunitySubmission
from invenio_rdm_records.services.permissions import (
    RDMRecordPermissionPolicy,
    RDMRequestsPermissionPolicy,
)
from invenio_records_permissions.generators import SystemProcess
from invenio_records_resources.services.files.generators import IfTransferType
from invenio_records_resources.services.files.transfer import LOCAL_TRANSFER_TYPE
from invenio_requests.services.generators import Creator, Receiver

from ..requests.curation import CurationRequest
from .generators import (
    CurationModerators,
    IfCurationRequestAccepted,
    IfCurationRequestExists,
    IfRequestTypes,
    TopicPermission,
)


class CurationRDMRecordPermissionPolicy(RDMRecordPermissionPolicy):
    """RDM record policy for curations."""

    can_preview = RDMRecordPermissionPolicy.can_preview + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]
    can_view = RDMRecordPermissionPolicy.can_view + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]
    can_read = RDMRecordPermissionPolicy.can_read + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]
    can_read_files = RDMRecordPermissionPolicy.can_read_files + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]

    # in order to get all base permissions in, we just add ours instead of adapting the then_ clause of the base permission
    can_get_content_files = RDMRecordPermissionPolicy.can_get_content_files + [
        IfTransferType(LOCAL_TRANSFER_TYPE, can_read_files),
        SystemProcess(),
    ]

    can_read_draft = RDMRecordPermissionPolicy.can_read_draft + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]
    can_draft_read_files = RDMRecordPermissionPolicy.can_draft_read_files + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]

    # in order to get all base permissions in, we just add ours instead of adapting the then_ clause of the base permission
    can_draft_get_content_files = (
        RDMRecordPermissionPolicy.can_draft_get_content_files
        + [
            IfTransferType(LOCAL_TRANSFER_TYPE, can_draft_read_files),
            SystemProcess(),
        ]
    )

    # in order to get all base permissions in, we just add ours instead of adapting the then_ clause of the base permission
    can_draft_media_get_content_files = (
        RDMRecordPermissionPolicy.can_draft_media_get_content_files
        + [
            IfTransferType(LOCAL_TRANSFER_TYPE, can_preview),
            SystemProcess(),
        ]
    )

    can_media_read_files = RDMRecordPermissionPolicy.can_media_read_files + [
        IfCurationRequestExists(then_=[CurationModerators()], else_=[]),
    ]
    can_media_get_content_files = (
        RDMRecordPermissionPolicy.can_media_get_content_files
        + [
            IfTransferType(LOCAL_TRANSFER_TYPE, can_read),
            SystemProcess(),
        ]
    )


class CurationRDMRequestsPermissionPolicy(RDMRequestsPermissionPolicy):
    """Customized permission policy for sane handling of curation requests."""

    curation_request_record_review = IfRequestTypes(
        [CurationRequest],
        then_=[TopicPermission(permission_name="can_review")],
        else_=[],
    )

    # Only allow community-submission requests to be accepted after the rdm-curation request has been accepted
    can_action_accept: Final = [
        IfRequestTypes(
            request_types=[CommunitySubmission],
            then_=[
                IfCurationRequestAccepted(
                    then_=RDMRequestsPermissionPolicy.can_action_accept,
                    else_=[],
                ),
            ],
            else_=RDMRequestsPermissionPolicy.can_action_accept,
        ),
    ]

    # Update can read and can comment with new states
    can_read: Final = [
        # Have to explicitly check the request type and circumvent using status, as creator/receiver will add a query filter where one entity must be the user.
        IfRequestTypes(
            [CurationRequest],
            then_=[
                Creator(),
                Receiver(),
                TopicPermission(permission_name="can_review"),
                SystemProcess(),
            ],
            else_=RDMRequestsPermissionPolicy.can_read,
        ),
    ]
    can_create_comment = can_read

    # Update submit to also allow record reviewers/managers for curation requests
    can_action_submit = RDMRequestsPermissionPolicy.can_action_submit + [
        curation_request_record_review,
    ]

    # Add new actions
    can_action_review = RDMRequestsPermissionPolicy.can_action_accept
    can_action_critique = RDMRequestsPermissionPolicy.can_action_accept
    can_action_resubmit = can_action_submit



# -*- coding: utf-8 -*-
#
# Copyright (C) 2021 CERN.
# Copyright (C) 2021-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Curations service configuration."""

from typing import Final

from invenio_records_resources.services import RecordServiceConfig
from invenio_records_resources.services.base.config import ConfiguratorMixin, FromConfig
from invenio_requests.services.requests.config import RequestSearchOptions

from . import facets
from .permissions import CurationRDMRequestsPermissionPolicy


class CurationsSearchOptions(RequestSearchOptions):
    """Search options."""

    facets: Final = {
        "type": facets.type,
        "status": facets.status,
    }


class CurationsServiceConfig(RecordServiceConfig, ConfiguratorMixin):
    """Curations service configuration."""

    # will use requests service for most calls.
    service_id = "curations"

    # common configuration
    permission_policy_cls = FromConfig(
        "REQUESTS_PERMISSION_POLICY",
        default=CurationRDMRequestsPermissionPolicy,
    )
    # TODO: update search options?
    search = CurationsSearchOptions



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""View functions for curations."""

from typing import cast

from flask import Blueprint, Flask


def create_curations_bp(app: Flask) -> Blueprint:
    """Create curations blueprint."""
    ext = app.extensions["invenio-curations"]
    return cast(Blueprint, ext.curations_resource.as_blueprint())



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Curations views module."""

from flask import Blueprint

from .api import create_curations_bp
from .ui import create_ui_blueprint

blueprint = Blueprint("invenio-curations-ext", __name__)

__all__ = (
    "blueprint",
    "create_curations_bp",
    "create_ui_blueprint",
)



# -*- coding: utf-8 -*-
#
# Copyright (C)      2023 CERN.
# Copyright (C) 2023-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Facet definitions."""

from invenio_i18n import gettext as _
from invenio_records_resources.services.records.facets import TermsFacet
from invenio_requests.services.requests import facets

type = TermsFacet(  # noqa: A001
    field="type",
    label=_("Type"),
    value_labels={
        **facets.type._value_labels,  # noqa: SLF001
        "rdm-curation": _("RDM Curation"),
    },
)

status = TermsFacet(
    field="status",
    label=_("Status"),
    value_labels={
        **facets.status._value_labels,  # noqa: SLF001
        "review": _("In review"),
        "critiqued": _("Changes requested"),
        "resubmitted": _("Resubmitted"),
    },
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Resources module."""

from .config import CurationsServiceConfig
from .service import CurationRequestService

__all__ = (
    "CurationRequestService",
    "CurationsServiceConfig",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Curations related generators."""

from collections.abc import Callable
from itertools import chain
from typing import Any, cast

from flask_principal import Need, RoleNeed
from invenio_access.permissions import Permission, system_identity
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_rdm_records.records.api import RDMDraft
from invenio_rdm_records.requests.entity_resolvers import RDMRecordProxy
from invenio_records_permissions.generators import ConditionalGenerator, Generator
from invenio_requests.customizations.request_types import RequestType
from invenio_requests.records.api import Request

from ..proxies import current_curations_service, unproxy
from .service import CurationRequestService


class IfRequestTypes(ConditionalGenerator):
    """Request-oriented generator checking for requests of certain types."""

    def __init__(
        self,
        request_types: list[type[RequestType]],
        then_: list[Generator],
        else_: list[Generator],
    ) -> None:
        """Constructor."""
        self.request_types = set(request_types)
        super().__init__(then_, else_)

    def _condition(self, request: Request | None = None, **__: Any) -> bool:
        """Check if the request type matches a configured type."""
        if request is not None:
            for request_type in self.request_types:
                if isinstance(request.type, request_type):
                    return True

        return False


class IfCurationRequestAccepted(ConditionalGenerator):
    """Request-oriented generator checking if a curation request has been accepted."""

    _curations_service: CurationRequestService = unproxy(current_curations_service)

    def __init__(
        self,
        record_access_func: Callable[
            [Request],
            RDMDraft,
        ] = lambda request: request.topic.resolve(),
        then_: list[Generator] | None = None,
        else_: list[Generator] | None = None,
    ) -> None:
        """Constructor."""
        self.record_access_func = record_access_func
        super().__init__(then_ or [], else_ or [])

    def _condition(self, request: Request | None = None, **__: Any) -> bool:
        """Check if the curation request for the record has been accepted."""
        if request is not None:
            record_to_curate = self.record_access_func(request)
            return (
                self._curations_service.accepted_record(
                    system_identity,
                    record_to_curate,
                )
                is not None
            )

        return False


class EntityReferenceServicePermission(Generator):
    """Request-oriented generator accessing a named permission from the entity's service config."""

    entity_field: str | None = None

    def __init__(self, permission_name: str, **__: Any) -> None:
        """Constructor specifying permission_name."""
        self.permission_name = permission_name
        assert self.entity_field is not None, "Subclass must define entity_field."
        super().__init__()

    def _get_permission(self, entity: RDMRecordProxy) -> list[Permission]:
        """Get the specified permission from the request entity service config."""
        permission_policy_cls = (
            entity.get_resolver().get_service().config.permission_policy_cls
        )

        return cast(
            list[Permission],
            getattr(permission_policy_cls, self.permission_name),
        )

    def _get_entity(self, request: Request) -> RDMRecordProxy:
        """Get the specified entity of the request."""
        return getattr(request, self.entity_field)  # type: ignore[arg-type]

    def needs(self, request: Request | None = None, **kwargs: Any) -> set[Need]:
        """Set of needs granting permission."""
        if request is None:
            return set()

        entity = self._get_entity(request)
        permission = self._get_permission(entity)
        try:
            record = entity.resolve()
        except PIDDoesNotExistError:
            # Could not resolve topic. This may happen when trying to serialize a request and checking its permissions.
            # The referenced entity could be deleted, which would result in not being able to serialize instead. Instead,
            # an empty set is returned for this permission.
            return set()
        popped_record = kwargs.pop("record")
        needs = [g.needs(record=record, **kwargs) for g in permission]

        kwargs["record"] = popped_record
        return set(chain.from_iterable(needs))

    def excludes(self, request: Request | None = None, **kwargs: Any) -> set[Need]:
        """Set of excludes denying permission."""
        if request is None:
            return set()

        entity = self._get_entity(request)
        permission = self._get_permission(entity)
        try:
            record = entity.resolve()
        except PIDDoesNotExistError:
            # Could not resolve topic. This may happen when trying to serialize a request and checking its permissions.
            # The referenced entity could be deleted, which would result in not being able to serialize instead. Instead,
            # an empty set is returned for this permission.
            return set()
        popped_record = kwargs.pop("record")
        excludes = [g.excludes(record=record, **kwargs) for g in permission]

        kwargs["record"] = popped_record
        return set(chain.from_iterable(excludes))


class TopicPermission(EntityReferenceServicePermission):
    """Request-oriented generator to get generators of specified permission name from the topic of the request."""

    entity_field: str = "topic"


class CurationModerators(Generator):
    """Permission generator that allows users with the `moderation` role."""

    _curations_service: CurationRequestService = unproxy(current_curations_service)

    def needs(self, **__: Any) -> list[Need]:
        """Allow access for the moderation role."""
        return [RoleNeed(self._curations_service.moderation_role_name)]


class IfCurationRequestExists(ConditionalGenerator):
    """Record-oriented generator checking if a curation request exists."""

    _curations_service: CurationRequestService = unproxy(current_curations_service)

    def _condition(self, record: RDMDraft | None = None, **__: Any) -> bool:
        """Check if the record has a curation request or not."""
        if record is not None:
            # We use the system identity here to avoid visibility issues
            request = self._curations_service.get_review(
                identity=system_identity,
                topic=record,
            )
            return request is not None

        return False



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Notification related utils for notifications."""

from __future__ import annotations

from typing import Any

from invenio_access.permissions import system_identity
from invenio_accounts.models import Role
from invenio_notifications.models import Notification, Recipient
from invenio_notifications.services.generators import RecipientGenerator
from invenio_records.dictutils import dict_lookup
from invenio_search.engine import dsl
from invenio_users_resources.proxies import current_users_service


class GroupMembersRecipient(RecipientGenerator):
    """Group/Role member recipient generator for notifications."""

    def __init__(self, key: str) -> None:
        """Initialize the generator with a key.

        Args:
            key: The context key to look up the group/role information

        """
        self.key = key

    def __call__(
        self,
        notification: Notification,
        recipients: dict[str, Recipient],
    ) -> dict[str, Recipient]:
        """Fetch group members as recipients.

        Args:
            notification: The notification object containing context
            recipients: Dictionary of existing recipients keyed by user ID

        Returns:
            Updated recipients dictionary with group members added

        """
        group: dict[str, Any] = dict_lookup(notification.context, self.key)

        # Group service does not contain information about users.
        role: Role = Role.query.filter(Role.id == group["id"]).one()

        user_ids: list[str] = [str(u.id) for u in role.users]

        if not user_ids:
            return recipients

        filter_: dsl.Q = dsl.Q("terms", **{"id": user_ids})
        users = current_users_service.scan(system_identity, extra_filter=filter_)
        for u in users:
            recipients[u["id"]] = Recipient(data=u)

        return recipients



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Module for request comments handling."""

import ast
from typing import Any, Final, cast

import dictdiffer
from flask import current_app
from invenio_i18n import lazy_gettext as _
from invenio_requests.proxies import current_events_service

from .diff import DIFF_TYPE, DiffProcessorBase
from .events import CurationCommentEventType


class CurationCommentError(Exception):
    """Custom exception for comment exceptions."""


class CommentProcessor:
    """Class for creating and updating curation request comments."""

    _comment_error: Final[dict] = {
        "field": "custom_fields.rdm-curation",
        "messages": [
            _("Record saved successfully, but failed to update request comment."),
        ],
    }

    def __init__(self, identity: Any, diff_processor: DiffProcessorBase) -> None:
        """Constructs."""
        self._identity = identity
        self._diff_processor = diff_processor

    def _validate_request(self, request: dict) -> bool:
        """Sanity check."""
        return request.get("is_open")  # type: ignore[return-value]

    def _is_comment_hit_created_by_curations_comment_processor(
        self,
        hit: dict,
    ) -> bool:
        """Check if a comment was automatically created by this processor."""
        type_ = hit.get("type")
        payload = hit.get("payload")
        return bool(type_ and payload and type_ == "C" and "reference_draft" in payload)

    def _cleanup_other_comments(
        self,
        events: list[dict],
    ) -> list[dict]:
        """Filter all comments created by users or by system but are not created by this processor."""
        return [
            hit
            for hit in events
            if self._is_comment_hit_created_by_curations_comment_processor(
                hit,
            )
        ]

    def _get_current_diffs(
        self,
        base_data: dict,
        new_data: dict,
    ) -> list[DIFF_TYPE] | None:
        """Return the diffs between old and new data.

        :param base_data: The data to compare against.
        :param new_data: New data.

        :returns: A list of dictdiffer.diff-style tuples.
        """
        diffs = dictdiffer.diff(base_data, new_data)
        diff_list = cast(list[DIFF_TYPE], list(diffs))

        if not diff_list:
            return None

        return diff_list

    def _create_new_comment(
        self,
        request: dict,
        content: str,
        reference_draft: str | None,
    ) -> None:
        """Build and send comment create event.

        :param request: The curation request.
        :param content: The actual comment.
        :param reference_draft: The draft to store for future updates.
        """
        payload = {"payload": {"content": content}}

        if reference_draft is not None:
            payload["payload"]["reference_draft"] = reference_draft

        try:
            current_events_service.create(
                self._identity,
                request["id"],
                payload,
                CurationCommentEventType(),
            )
        except Exception as e:  # noqa: BLE001
            # TODO: revise the exception handling for comment feature
            current_app.logger.warning(e, exc_info=True)

    def _update_existing_comment(
        self,
        new_data: str,
        crt_comment_event: dict,
    ) -> None:
        """Build and send comment update event.

        :param new_data: The updated comment.
        :param crt_comment_event: The existing comment.
        """
        if crt_comment_event is None:
            raise CurationCommentError

        payload = {"payload": {"content": new_data}}

        try:
            current_events_service.update(
                self._identity,
                crt_comment_event.get("id"),
                payload,
                revision_id=crt_comment_event.get("revision_id"),
            )
        except Exception as e:  # noqa: BLE001
            # TODO: revise the exception handling for comment feature
            current_app.logger.warning(e, exc_info=True)

    def _handle_critiqued_resubmit_status(
        self,
        request: dict,
        current_draft: dict,
        new_data: dict,
        msg: str,
        errors: list[dict] | None,
    ) -> None:
        """Handle the draft save when request is in status critiqued or resubmitted.

         Flow:
            Update draft while critiqued without resubmitting immediately: create
            comment and then update the same comment if new updates of this sort come.
         OR
             Update draft while request is in resubmitted status. The same principle
             applies: each time update the last comment or create a new one if none
             is found.

        :param request: The curation request.
        :param current_draft: The state of the draft used to compare against to get the changes.
        :param new_data: Latest state of the record.
        :param msg: Flag to differentiate between request states.
        :param errors: Add to errors list to display a message if something bad happens.
        """
        last_event = list(
            current_events_service.search(self._identity, request["id"]).hits,
        )[-1]

        # if last event is an action OR a comment created manually by the user
        if last_event.get(
            "type",
        ) == "L" or not self._is_comment_hit_created_by_curations_comment_processor(
            last_event,
        ):
            self._create_comment_with_latest_changes(
                request,
                current_draft,
                new_data,
                msg,
                errors,
            )
        # if last event is a comment generated by this processor
        elif self._is_comment_hit_created_by_curations_comment_processor(last_event):
            self._compute_diff_and_update_event(last_event, new_data, msg, errors)

    def _compute_diff_and_update_event(
        self,
        event: dict,
        new_data: dict,
        msg: str,
        errors: list[dict] | None,  # noqa: ARG002
    ) -> None:
        """Compute diff between 2 draft states and update the comment with the result.

        :param event: The comment event to update.
        :param new_data: Latest state of the record.
        :param msg: Flag to differentiate between request states.
        :param errors: Add to errors list to display a message if something bad happens.
        """
        payload = event.get("payload")
        if payload is None:
            msg = "Got empty payload from event"
            raise CurationCommentError(msg)

        ref_draft = payload.get("reference_draft")
        if ref_draft is None:
            msg = "Got emtpy reference draft flag from event payload"
            raise CurationCommentError(msg)

        base_draft = ast.literal_eval(ref_draft)
        diffs = self._get_current_diffs(base_draft, new_data)
        if diffs is None:
            return
        self._diff_processor.map_and_build_diffs(diffs)

        self._update_existing_comment(
            self._diff_processor.to_html(msg),
            event,
        )

    def _create_comment_with_latest_changes(
        self,
        request: dict,
        current_draft: dict,
        new_data: dict,
        msg: str,
        errors: list[dict] | None,  # noqa: ARG002
        reference_draft: bool = True,  # noqa: FBT001, FBT002
    ) -> None:
        """Compute diff between 2 draft states and create comment with the result.

        :param request: The curation request.
        :param current_draft: The state of the draft used to compare against to get the changes.
        :param new_data: Latest state of the record.
        :param msg: Flag to differentiate between request states.
        :param errors: Add to errors list to display a message if something bad happens.
        :param reference_draft: Flag that can be unset to omit the storing of a reference draft.
        """
        diffs = self._get_current_diffs(current_draft, new_data)
        if diffs is None:
            return
        self._diff_processor.map_and_build_diffs(diffs)

        self._create_new_comment(
            request,
            self._diff_processor.to_html(msg),
            str(current_draft) if reference_draft else None,
        )

    def process_comment(
        self,
        request: dict,
        new_data: dict,
        current_draft: dict,
        errors: list[dict] | None,
    ) -> None:
        """Dispatch the handling of request comments based on request status.

        :param request: The curation request.
        :param new_data: Latest state of the record.
        :param current_draft: The state of the draft used to compare against to get the changes.
        :param errors: Add to errors list to display a message if something bad happens.
        """
        if not self._validate_request(request):
            errors.append(self._comment_error)  # type: ignore[union-attr]
            return

        try:
            if request["status"] == "resubmitted":
                self._handle_critiqued_resubmit_status(
                    request,
                    current_draft,
                    new_data,
                    "resubmit",
                    errors,
                )

            elif request["status"] == "critiqued":
                self._handle_critiqued_resubmit_status(
                    request,
                    current_draft,
                    new_data,
                    "update_while_critiqued",
                    errors,
                )

            elif request["status"] == "review":
                # on review status, just create comments with new diffs, hopefully to notify the curator more easily
                self._create_comment_with_latest_changes(
                    request,
                    current_draft,
                    new_data,
                    "update_while_review",
                    errors,
                    reference_draft=False,
                )

        except Exception as e:  # noqa: BLE001
            # fail-safe in case of any unexpected error
            # TODO: improve error handling in this workflow
            current_app.logger.warning(e, exc_info=True)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Invenio curation module for rdm-records."""

from setuptools import setup

setup()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curation is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Curation request type."""

from __future__ import annotations

from typing import ClassVar

from flask_principal import Identity
from invenio_i18n import lazy_gettext as _
from invenio_notifications.services.uow import NotificationOp
from invenio_records_resources.services.uow import UnitOfWork
from invenio_requests.customizations import RequestState, RequestType, actions
from invenio_requests.customizations.actions import RequestAction

from invenio_curations.notifications.builders import (
    CurationRequestAcceptNotificationBuilder,
    CurationRequestCritiqueNotificationBuilder,
    CurationRequestResubmitNotificationBuilder,
    CurationRequestReviewNotificationBuilder,
    CurationRequestSubmitNotificationBuilder,
)


class CurationCreateAndSubmitAction(actions.CreateAndSubmitAction):
    """Create and submit a request."""

    def execute(self, identity: Identity, uow: UnitOfWork) -> None:
        """Execute the create & submit action."""
        uow.register(
            NotificationOp(
                CurationRequestSubmitNotificationBuilder.build(
                    identity=identity,
                    request=self.request,
                ),
            ),
        )

        super().execute(identity, uow)


class CurationSubmitAction(actions.SubmitAction):
    """Submit action for user access requests."""

    # list of statuses this action can be performed from
    status_from: ClassVar[list[str]] = ["created"]

    def execute(self, identity: Identity, uow: UnitOfWork) -> None:
        """Execute the submit action."""
        uow.register(
            NotificationOp(
                CurationRequestSubmitNotificationBuilder.build(
                    identity=identity,
                    request=self.request,
                ),
            ),
        )
        super().execute(identity, uow)


class CurationAcceptAction(actions.AcceptAction):
    """Accept a request."""

    # Require to go through review before accepting.
    status_from: ClassVar[list[str]] = ["review"]

    def execute(self, identity: Identity, uow: UnitOfWork) -> None:
        """Execute the accept action."""
        uow.register(
            NotificationOp(
                CurationRequestAcceptNotificationBuilder.build(
                    identity=identity,
                    request=self.request,
                ),
            ),
        )

        super().execute(identity, uow)


class CurationDeclineAction(actions.DeclineAction):
    """Decline a request."""

    # Instead of declining, the record should be critiqued.
    status_from: ClassVar[list[str]] = []


class CurationCancelAction(actions.CancelAction):
    """Cancel a request."""

    # A user might want to cancel their request.
    # Also done when a draft for an already published record is deleted/discarded
    status_from: ClassVar[list[str]] = [
        "accepted",
        "cancelled",
        "created",
        "critiqued",
        "declined",
        "expired",
        "resubmitted",
        "review",
        "submitted",
    ]


class CurationExpireAction(actions.ExpireAction):
    """Expire a request."""

    status_from: ClassVar[list[str]] = ["submitted", "critiqued", "resubmitted"]


class CurationDeleteAction(actions.DeleteAction):
    """Delete a request."""

    # When a user deletes their draft, the request will get deleted. Should be possible from every state.
    # Usually delete is only possible programmatically, as the base permissions allow user driven deletion
    # only during `created` status
    status_from: ClassVar[list[str]] = [
        "accepted",
        "cancelled",
        "created",
        "critiqued",
        "declined",
        "expired",
        "resubmitted",
        "review",
        "submitted",
    ]


class CurationReviewAction(actions.RequestAction):
    """Mark request as review."""

    status_from: ClassVar[list[str]] = ["submitted", "resubmitted"]
    status_to: ClassVar[str] = "review"

    def execute(self, identity: Identity, uow: UnitOfWork) -> None:
        """Execute the review action."""
        uow.register(
            NotificationOp(
                CurationRequestReviewNotificationBuilder.build(
                    identity=identity,
                    request=self.request,
                ),
            ),
        )

        super().execute(identity, uow)


class CurationCritiqueAction(actions.RequestAction):
    """Request changes for request."""

    status_from: ClassVar[list[str]] = ["review"]
    status_to: ClassVar[str] = "critiqued"

    def execute(self, identity: Identity, uow: UnitOfWork) -> None:
        """Execute the critique action."""
        uow.register(
            NotificationOp(
                CurationRequestCritiqueNotificationBuilder.build(
                    identity=identity,
                    request=self.request,
                ),
            ),
        )

        super().execute(identity, uow)


class CurationResubmitAction(actions.RequestAction):
    """Mark request as ready for review."""

    status_from: ClassVar[list[str]] = [
        "critiqued",
        "accepted",
        "cancelled",
        "declined",
    ]
    status_to: ClassVar[str] = "resubmitted"

    def execute(self, identity: Identity, uow: UnitOfWork) -> None:
        """Execute the resubmit action."""
        uow.register(
            NotificationOp(
                CurationRequestResubmitNotificationBuilder.build(
                    identity=identity,
                    request=self.request,
                ),
            ),
        )
        super().execute(identity, uow)


#
# Request
#
class CurationRequest(RequestType):
    """Curation request type."""

    type_id: ClassVar[str] = "rdm-curation"
    name: ClassVar[str] = _("Curation")

    # Dict mapping action names to action classes
    available_actions: ClassVar[dict[str, type[RequestAction]]] = {
        **RequestType.available_actions,
        "create": CurationCreateAndSubmitAction,
        "submit": CurationSubmitAction,
        "accept": CurationAcceptAction,
        "decline": CurationDeclineAction,
        "cancel": CurationCancelAction,
        "expire": CurationExpireAction,
        "delete": CurationDeleteAction,
        "review": CurationReviewAction,
        "critique": CurationCritiqueAction,
        "resubmit": CurationResubmitAction,
    }

    # Dict mapping status names to RequestState values
    available_statuses: ClassVar[dict[str, RequestState]] = {
        **RequestType.available_statuses,
        "review": RequestState.OPEN,
        "critiqued": RequestState.OPEN,
        "resubmitted": RequestState.OPEN,
    }
    """Available statuses for the request.

    The keys in this dictionary is the set of available statuses, and their
    values are indicators whether this request is considered to be open, closed
    or undefined.
    """

    create_action: ClassVar[str] = "create"
    """Defines the action that's able to create this request.

    This must be set to one of the available actions for the custom request type.
    """

    creator_can_be_none: ClassVar[bool] = False
    topic_can_be_none: ClassVar[bool] = False
    allowed_creator_ref_types: ClassVar[list[str]] = ["user", "community"]
    allowed_receiver_ref_types: ClassVar[list[str]] = ["group"]
    allowed_topic_ref_types: ClassVar[list[str]] = ["record"]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Notification related utils for notifications."""

from __future__ import annotations

from typing import ClassVar

from flask_principal import Identity
from invenio_notifications.models import Notification
from invenio_notifications.registry import EntityResolverRegistry
from invenio_notifications.services.builders import NotificationBuilder
from invenio_notifications.services.filters import RecipientFilter
from invenio_notifications.services.generators import (
    ContextGenerator,
    EntityResolve,
    RecipientGenerator,
    UserEmailBackend,
)
from invenio_requests.notifications.filters import UserRecipientFilter
from invenio_requests.records.api import Request
from invenio_users_resources.notifications.filters import UserPreferencesRecipientFilter
from invenio_users_resources.notifications.generators import UserRecipient

from .generators import GroupMembersRecipient


class CurationRequestActionNotificationBuilder(NotificationBuilder):
    """Notification builder for curation actions."""

    type: ClassVar[str] = "curation-request"

    context: ClassVar[list[ContextGenerator]] = [
        EntityResolve(key="request"),
        EntityResolve(key="request.created_by"),
        EntityResolve(key="request.topic"),
        EntityResolve(key="request.receiver"),
        EntityResolve(key="executing_user"),
    ]

    # recipientGenerator: class for all recipient finder classes
    # (e.g., UserRecipient, GroupMembersRecipient) - defines who gets notifications
    recipients: ClassVar[list[RecipientGenerator]] = []

    recipient_filters: ClassVar[list[RecipientFilter]] = [
        UserPreferencesRecipientFilter(),
        UserRecipientFilter("executing_user"),
    ]

    recipient_backends: ClassVar[list[UserEmailBackend]] = [
        UserEmailBackend(),
    ]

    @classmethod
    def build(cls, identity: Identity, request: Request) -> Notification:
        """Build notification with request context.

        Args:
            identity: The identity performing the action
            request: The curation request

        Returns:
            Notification object with proper context

        """
        return Notification(
            type=cls.type,
            context={
                "executing_user": EntityResolverRegistry.reference_identity(identity),
                "request": EntityResolverRegistry.reference_entity(request),
            },
        )


class CurationRequestSubmitNotificationBuilder(
    CurationRequestActionNotificationBuilder,
):
    """Notification builder for submit action."""

    type: ClassVar[str] = f"{CurationRequestActionNotificationBuilder.type}.submit"
    recipients: ClassVar[list[RecipientGenerator]] = [
        GroupMembersRecipient("request.receiver"),
    ]


class CurationRequestResubmitNotificationBuilder(
    CurationRequestActionNotificationBuilder,
):
    """Notification builder for resubmit action."""

    type: ClassVar[str] = f"{CurationRequestActionNotificationBuilder.type}.resubmit"
    recipients: ClassVar[list[RecipientGenerator]] = [
        GroupMembersRecipient("request.receiver"),
    ]


class CurationRequestReviewNotificationBuilder(
    CurationRequestActionNotificationBuilder,
):
    """Notification builder for review action."""

    type: ClassVar[str] = f"{CurationRequestActionNotificationBuilder.type}.review"
    recipients: ClassVar[list[RecipientGenerator]] = [
        UserRecipient("request.created_by"),
    ]


class CurationRequestAcceptNotificationBuilder(
    CurationRequestActionNotificationBuilder,
):
    """Notification builder for accept action."""

    type: ClassVar[str] = f"{CurationRequestActionNotificationBuilder.type}.accept"
    recipients: ClassVar[list[RecipientGenerator]] = [
        UserRecipient("request.created_by"),
    ]


class CurationRequestCritiqueNotificationBuilder(
    CurationRequestActionNotificationBuilder,
):
    """Notification builder for critique action."""

    type: ClassVar[str] = f"{CurationRequestActionNotificationBuilder.type}.critique"
    recipients: ClassVar[list[RecipientGenerator]] = [
        UserRecipient("request.created_by"),
    ]



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 CERN.
#
# Invenio-Curations is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Configuration helper for React-SearchKit."""

from collections.abc import Callable
from functools import partial
from typing import Any

from flask import current_app
from invenio_search_ui.searchconfig import search_app_config


def search_app_context() -> dict[str, Callable[..., dict[str, Any]]]:
    """Search app context processor."""
    return {
        "search_app_curations_requests_config": partial(
            search_app_config,
            config_name="CURATIONS_SEARCH_REQUESTS",
            available_facets=current_app.config["REQUESTS_FACETS"],
            sort_options=current_app.config["RDM_SORT_OPTIONS"],
            endpoint="/api/curations/",
            headers={"Accept": "application/json"},
            initial_filters=[["is_open", "true"]],
            hidden_params=[["expand", "1"]],
        ),
    }



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# Invenio-Curations is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Record curation service errors."""

from invenio_i18n import lazy_gettext as _


class OpenRecordCurationRequestAlreadyExistsError(Exception):
    """An open request already exists for the record."""

    description = _("An open curation request for this record exists already.")


class CurationRequestNotAcceptedError(Exception):
    """No curation request exists yet."""

    description = _("The record has not been curated yet.")


class RoleNotFoundError(Exception):
    """The role does not exist."""

    def __init__(self, role_name: str) -> None:
        """Initialise error."""
        super().__init__(
            _("A role with name '%(name)s' does not exist.", name=role_name),
        )
